@@ -103,13 +103,15 @@ class InterfaceViewer(ViewportPlugin):
         section.disable()
 
     pyatspi.Registry.registerEventListener(
-        self.onAccNameChanged, 'object:property-change:accessible-name')
+        self.onAccNameOrRoleChanged, 'object:property-change:accessible-name',
+        'object:property-change:accessible-role')
 
-  def onAccNameChanged(self, event):
+  def onAccNameOrRoleChanged(self, event):
     '''
-    Listener for accessible name changes, if it is ours, change the name.
+    Listener for accessible name/role changes, if it is ours, change the name.
 
-    @param event: 'object:property-change:accessible-name' event.
+    @param event: 'object:property-change:accessible-name' or
+                  'object:property-change:accessible-role' event.
     @type acc: Accessibility.Event
     '''
     if event.source != self.node.acc:
@@ -312,6 +314,7 @@ class _SectionAccessible(_InterfaceSection):
     self.desc_label = ui_xml.get_object('label_acc_desc')
     self.help_text_label = ui_xml.get_object('label_acc_help_text')
     self.id_label = ui_xml.get_object('label_acc_id')
+    self.locale_label = ui_xml.get_object('label_locale')
     self.process_id_label = ui_xml.get_object('label_process_id')
 
     # configure states tree view
@@ -350,6 +353,7 @@ class _SectionAccessible(_InterfaceSection):
         self.id_label.set_label(acc.accessibleId)
     except:
         self.id_label.set_label(_('(no ID)'))
+    self.locale_label.set_label(acc.get_object_locale() or '')
     self.process_id_label.set_text(str(acc.get_process_id()))
 
     states = [pyatspi.stateToString(s) for s in acc.getState().getStates()]
@@ -564,6 +568,8 @@ class _SectionApplication(_InterfaceSection):
   @type label_tk: gtk.Label
   @ivar label_version: Label for toolkit version.
   @type label_version: gtk.Label
+  @ivar label_atspi_version: Label for AT-SPI IPC version.
+  @type label_atspi_version: gtk.Label
   '''
   interface_name = 'Application'
   def init(self, ui_xml):
@@ -577,6 +583,7 @@ class _SectionApplication(_InterfaceSection):
     self.label_id = ui_xml.get_object('label_app_id')
     self.label_tk = ui_xml.get_object('label_app_tk')
     self.label_version = ui_xml.get_object('label_app_version')
+    self.label_atspi_version = ui_xml.get_object('label_atspi_version')
 
   def populateUI(self, acc):
     '''
@@ -586,10 +593,10 @@ class _SectionApplication(_InterfaceSection):
     @param acc: The currently selected accessible.
     @type acc: Accessibility.Accessible
     '''
-    ai = acc.queryApplication()
-    self.label_id.set_text(repr(ai.id))
-    self.label_tk.set_text(ai.toolkitName)
-    self.label_version.set_text(ai.version)
+    self.label_id.set_text(repr(acc.get_id()))
+    self.label_tk.set_text(acc.get_toolkit_name())
+    self.label_version.set_text(acc.get_toolkit_version())
+    self.label_atspi_version.set_text(acc.get_atspi_version() or '')
 
   def clearUI(self):
     '''
@@ -598,6 +605,7 @@ class _SectionApplication(_InterfaceSection):
     self.label_id.set_text('')
     self.label_tk.set_text('')
     self.label_version.set_text('')
+    self.label_atspi_version.set_text('')
 
 class _SectionComponent(_InterfaceSection):
   '''
