@@ -6,8 +6,8 @@ AT-SPI validation plugin.
 @copyright: Copyright (c) 2007 IBM Corporation
 @license: BSD
 
-All rights reserved. This program and the accompanying materials are made 
-available under the terms of the BSD which accompanies this distribution, and 
+All rights reserved. This program and the accompanying materials are made
+available under the terms of the BSD which accompanies this distribution, and
 is available at U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 import gi
@@ -20,7 +20,7 @@ import os
 import traceback
 import sys
 import glob
-import imp
+import importlib
 import webbrowser
 from accerciser.plugin import ViewportPlugin
 from accerciser.i18n import _, N_, DOMAIN
@@ -29,7 +29,7 @@ import pyatspi
 UI_FILE = os.path.join(os.path.dirname(__file__), 'validate.ui')
 USER_SCHEMA_PATH = os.path.join(GLib.get_user_data_dir(), 'accerciser',
                                 'plugindata', 'validate')
-SYS_SCHEMA_PATH = os.path.join(sys.prefix, 'share', 'accerciser', 
+SYS_SCHEMA_PATH = os.path.join(sys.prefix, 'share', 'accerciser',
                                'plugindata', 'validate')
 VALIDATORS = {}
 SCHEMA_METADATA = {}
@@ -68,22 +68,24 @@ class ValidatorManager(type):
     '''
     for path in [USER_SCHEMA_PATH, SYS_SCHEMA_PATH]:
       for fn in glob.glob(os.path.join(path, '*.py')):
-        module = os.path.basename(fn)[:-3]
-        params = imp.find_module(module, [path])
-        schema = imp.load_module(module, *params)
+        module_name = os.path.basename(fn)[:-3]
+        spec = importlib.util.spec_from_file_location(module_name, fn)
+        module = importlib.util.module_from_spec(spec)
+        spec.loader.exec_module(module)
+
         try:
           # try to get descriptive fields from the module
-          SCHEMA_METADATA[module] = schema.__metadata__
+          SCHEMA_METADATA[module_name] = module.__metadata__
         except AttributeError:
           # default to usinf file name as description
-          SCHEMA_METADATA[module] = {'name' : module,
-                                        'description' : _('No description')}
-      
+          SCHEMA_METADATA[module_name] = {'name' : module,
+                                          'description' : _('No description')}
+
   @staticmethod
   def getValidators(name):
     '''
     Gets all validator classes within a schema.
-    
+
     @param name: Name of a schema
     @return: List of Validator objects
     @raise KeyError: When the schema is not known
@@ -111,12 +113,12 @@ class ValidatorManager(type):
 
 class Validator(with_metaclass(ValidatorManager, object)):
   '''
-  Base class for all validators. Defines the public interface used by the 
+  Base class for all validators. Defines the public interface used by the
   plug-in controller/view to generate validation reports.
   '''
   def __init__(self):
     pass
-  
+
   def condition(self, acc):
     '''
     Checks if this validator is fit to test the given accessible. For instance,
@@ -127,7 +129,7 @@ class Validator(with_metaclass(ValidatorManager, object)):
     @raise Exception: Same as returning False
     '''
     return True
-  
+
   def before(self, acc, state, view):
     '''
     Tests the accessible before testing its descendants.
@@ -138,7 +140,7 @@ class Validator(with_metaclass(ValidatorManager, object)):
     @param view: View object to use to log test results
     '''
     pass
-  
+
   def after(self, acc, state, view):
     '''
     Tests the accessible after testing its descendants.
@@ -197,6 +199,7 @@ class ValidatorViewport(ViewportPlugin):
     self.clear = self.main_xml.get_object('clear button')
     self.schema = self.main_xml.get_object('schema combo')
     self.validator_buffer = gtk.TextBuffer()
+    self.idle_validate_id = None
 
     # model for the combobox
     model = gtk.ListStore(GObject.TYPE_STRING, GObject.TYPE_STRING)
@@ -208,7 +211,7 @@ class ValidatorViewport(ViewportPlugin):
       d = vm.getSchemaMetadata(name)
       model.append(['%s - %s' % (d['name'], d['description']), name])
     self.schema.set_active(0)
-    
+
     # model for the report
     model = gtk.ListStore(str, str, object, str)
     self.report.set_model(model)
@@ -217,7 +220,7 @@ class ValidatorViewport(ViewportPlugin):
     cell = gtk.CellRendererText()
     self.schema.pack_start(cell, True)
     self.schema.add_attribute(cell, 'text', 0)
-    
+
     # log level column
     col = gtk.TreeViewColumn(_('Level'))
     rend = gtk.CellRendererText()
@@ -231,10 +234,10 @@ class ValidatorViewport(ViewportPlugin):
 
     # set progress bar to zero initially
     self.progress.set_fraction(0.0)
-        
+
     self.main_xml.connect_signals(self)
     self.show_all()
-    
+
   def onAccChanged(self, acc):
     '''
     Stops validation if the accessible hierarchy changes.
@@ -243,7 +246,7 @@ class ValidatorViewport(ViewportPlugin):
     '''
     if self.walk is not None:
       self._stopValidate()
-    
+
   def _onValidate(self, widget):
     '''
     Starts or stops a validation run.
@@ -280,7 +283,7 @@ class ValidatorViewport(ViewportPlugin):
     n_rows_to_write = 5
     if n_rows_to_write > remaining_rows:
       n_rows_to_write = remaining_rows
-  
+
     file_str_list = [] # list to store strings to be written to file
     start = self.curr_file_row
     end = (self.curr_file_row + n_rows_to_write)
@@ -297,7 +300,7 @@ class ValidatorViewport(ViewportPlugin):
       # add url role to buffer
       file_str_list.append("%s: %s\n\n" % (_('Hyperlink'), val[3]))
       self.curr_file_row += 1
-  
+
     self.save_to.write(''.join(file_str_list))
 
     return True
@@ -308,16 +311,15 @@ class ValidatorViewport(ViewportPlugin):
 
     @param button: Save button
     '''
-    save_dialog = gtk.FileChooserDialog(
-      'Save validator output',
-      action=gtk.FileChooserAction.SAVE,
-      buttons=(gtk.STOCK_CANCEL, gtk.ResponseType.CANCEL,
-               gtk.STOCK_OK, gtk.ResponseType.OK))
-    #save_dialog.connect("response", self._savedDiagResponse)
+    save_dialog = gtk.FileChooserNative.new(
+      _('Save validator output'),
+      self.get_toplevel(),
+      gtk.FileChooserAction.SAVE,
+      _('_OK'),
+      _('_Cancel'))
     save_dialog.set_do_overwrite_confirmation(True)
-    save_dialog.set_default_response(gtk.ResponseType.OK)
     response = save_dialog.run()
-    if response == gtk.ResponseType.OK:
+    if response == gtk.ResponseType.ACCEPT:
       self.save_to = open(save_dialog.get_filename(), 'w')
       GLib.idle_add(self._writeFile)
     save_dialog.destroy()
@@ -386,8 +388,8 @@ class ValidatorViewport(ViewportPlugin):
     Ready appropriate variables for a save
     '''
     self.curr_file_row = 0
-    self.n_report_rows = 0 
-    self.row_values = [] 
+    self.n_report_rows = 0
+    self.row_values = []
 
   def _startValidate(self):
     '''
@@ -398,6 +400,9 @@ class ValidatorViewport(ViewportPlugin):
     self.report.get_model().clear()
     # get the validators
     index = self.schema.get_active()
+    if index == -1:
+      self.validate.set_active(False)
+      return
     row = self.schema.get_model()[index]
     self.vals = ValidatorManager.getValidators(row[1])
     # build a new state dict
@@ -418,6 +423,8 @@ class ValidatorViewport(ViewportPlugin):
     Stops a validation run by disabling the idle callback and restoring the
     various UI components to their enabled states.
     '''
+    if self.idle_validate_id == None:
+      return
     # stop callbacks
     GLib.source_remove(self.idle_validate_id)
     # destroy generator
@@ -432,7 +439,7 @@ class ValidatorViewport(ViewportPlugin):
     self.help.set_sensitive(True)
     self.save.set_sensitive(True)
     self.clear.set_sensitive(True)
-     
+
   def _onValidateIdle(self):
     '''
     Tests one accessible at a time on each idle callback by advancing the
@@ -471,14 +478,14 @@ class ValidatorViewport(ViewportPlugin):
       # recurse
       gen_traverse = self._traverse(child, state)
       while 1:
-        # yield before continuing processing 
+        # yield before continuing processing
         yield None
         try:
           # get one descendant
           next(gen_traverse)
         except StopIteration:
           break
-    
+
   def _genAccessible(self, acc, state):
     '''
     Tests the given accessible in the before pass if its test condition is
@@ -552,11 +559,11 @@ class ValidatorViewport(ViewportPlugin):
       acc = model[iter][2]
       if acc:
         self.node.update(acc)
-                             
+
   def _exceptionError(self, acc, ex):
     '''
     Logs an unexpected exception that occurred during execution of a validator.
-    
+
     @param acc: Accessible under test when the exception occurred
     @param ex: The exception
     '''
@@ -571,7 +578,7 @@ class ValidatorViewport(ViewportPlugin):
     '''
     level = _('ERROR')
     self.report.get_model().append([level, text, acc, url])
-    
+
   def warn(self, text, acc, url=''):
     '''
     Used by validators to log warning messages for accessibility problems that
@@ -579,14 +586,14 @@ class ValidatorViewport(ViewportPlugin):
     '''
     level = _('WARN')
     self.report.get_model().append([level, text, acc, url])
-  
+
   def info(self, text, acc, url=''):
     '''
     Used by validators to log informational messages.
     '''
     level = _('INFO')
     self.report.get_model().append([level, text, acc, url])
-    
+
   def debug(self, text, acc, url=''):
     '''
     Used by validators to log debug messages.
