@@ -1,4 +1,5 @@
 from gi.repository import GLib
+from gi.repository import Gio as gio
 from gi.repository import Gtk as gtk
 from gi.repository import Atk as atk
 
@@ -8,7 +9,7 @@ from .i18n import _
 from pyatspi import getPath
 from random import random
 import random
-from . import ui_manager
+from . import menus
 
 COL_NAME = 0
 COL_APP = 1
@@ -25,23 +26,24 @@ class BookmarkStore(gtk.ListStore):
   2. Persists bookmark changes to disk.
   3. Keeps bookmarks submenu up to date.
 
-  @ivar _bookmarks_action_group: Bookmarks' action group.
-  @type _bookmarks_action_group: gtk.ActionGroup
+  @ivar application: The accerciser application.
+  @type application: gtk.Application
   @ivar node: Main application's node.
   @type node: L{Node}
   @ivar _xmldoc: XML documenr object.
   @type _xmldoc: xml.dom.DOMImplementation
   '''
-  def __init__(self, node, window):
+  def __init__(self, node, application, window):
     '''
     Initialize bookmark manager. Load saved bookmarks from disk.
-    
+
     @param node: Main application's node.
     @type node: L{Node>
+    @ivar application: The accerciser application.
+    @type application: gtk.Application
     '''
     gtk.ListStore.__init__(self, object)
-    self._bookmarks_action_group = gtk.ActionGroup.new('BookmarkActions')
-    ui_manager.uimanager.insert_action_group(self._bookmarks_action_group, 0)
+    self.application = application
     self._buildMenuUI()
     self.node = node
     self.parent_window = window
@@ -56,38 +58,24 @@ class BookmarkStore(gtk.ListStore):
     self.connect('row-changed', self._onRowChanged)
     self.connect('row-deleted', self._onRowDeleted)
     self.connect('row-inserted', self._onRowInserted)
-    
+
   def _buildMenuUI(self):
     '''
     Build's the initial submenu with functionality menu items.
     '''
-    self._bookmarks_action_group.add_actions(
-      [('AddBookmark', gtk.STOCK_ADD, 
-        _('_Add Bookmark…'), '<Control>d',
-        _('Bookmark selected accessible.'), self._onAddBookmark),
-       ('EditBookmarks', gtk.STOCK_EDIT, 
-        _('_Edit Bookmarks…'), None,
-        _('Manage bookmarks.'), self._onEditBookmarks)])
-
-
-    for action in self._bookmarks_action_group.list_actions():
-      merge_id = ui_manager.uimanager.new_merge_id()
-      action_name = action.get_name()
-      ui_manager.uimanager.add_ui(merge_id, ui_manager.BOOKMARKS_MENU_PATH, 
-                            action_name, action_name, 
-                            gtk.UIManagerItemType.MENUITEM, False)
-
-    ui_manager.uimanager.add_ui(ui_manager.uimanager.new_merge_id(), 
-                                ui_manager.BOOKMARKS_MENU_PATH, 
-                                'sep', None, 
-                                gtk.UIManagerItemType.SEPARATOR, False)
+    menu_items = [
+      (menus.bookmarks_menu_general_section, 'add_bookmark', 'list-add',
+        _('_Add Bookmark…'), '<Control>d', self._onAddBookmark),
+      (menus.bookmarks_menu_general_section,  'edit_bookmarks', 'document-edit',
+        _('_Edit Bookmarks…'), None, self._onEditBookmarks)]
+    self.application.addMenuItems(menu_items)
 
   def _onAddBookmark(self, action, data=None):
     '''
     Callback for AddBookmark action.
-    
+
     @param action: Action that emitted this signal.
-    @type action: gtk.Action
+    @type action: gio.Action
     '''
     iter = self.bookmarkCurrent()
     if not iter: return
@@ -97,15 +85,15 @@ class BookmarkStore(gtk.ListStore):
     if response_id == gtk.ResponseType.OK:
       bookmark.title, bookmark.app, bookmark.path = dialog.getFields()
     else:
-      self.remove(iter)
+      self.removeBookmark(bookmark)
     dialog.destroy()
 
   def _onEditBookmarks(self, action, data=None):
     '''
     Callback for EditBookmark action.
-    
+
     @param action: Action that emitted this signal.
-    @type action: gtk.Action
+    @type action: gio.Action
     '''
     dialog = self._EditDialog(self)
     dialog.show()
@@ -132,51 +120,40 @@ class BookmarkStore(gtk.ListStore):
   def addBookmark(self, title, app, path):
     '''
     Add a bookmark to the maanger.
-    
+
     @param title: Title of bookmark
     @type title: string
     @param app: Application name of bookmark.
     @type app: string
     @param path: Path of bookmarks.
     @type path: string
-    
+
     @return: Tree iter of new bookmark.
     @rtype: gtk.TreeIter
     '''
     iter = self.append([None])
-    merge_id = ui_manager.uimanager.new_merge_id()
-    name = 'Bookmark%s' % merge_id
-    bookmark = self._Bookmark(name, title, app, path, merge_id)
-    bookmark.connect('activate', self._onBookmarkActivate)
-    bookmark.connect('notify', self._onBookmarkChanged)
-    self._bookmarks_action_group.add_action(bookmark)
-    ui_manager.uimanager.add_ui(merge_id, 
-                                '/MainMenuBar/Bookmarks', name, name, 
-                                gtk.UIManagerItemType.MENUITEM, False)
+    bookmark = self._Bookmark(self, title, app, path, self.application)
     self[iter][0] = bookmark
     return iter
 
   def removeBookmark(self, bookmark):
     '''
     Remove bookmark from manager.
-    
+
     @param bookmark: Bookmark to remove.
     @type bookmark: BookmarkStore._Bookmark
     '''
-    self._bookmarks_action_group.remove_action(bookmark)
-    ui_manager.uimanager.remove_ui(bookmark.merge_id)
+    bookmark.removeSelf()
     for row in self:
       if row[0] == bookmark:
         self.remove(row.iter)
 
-  def _onBookmarkChanged(self, bookmark, property):
+  def _onBookmarkChanged(self, bookmark):
     '''
-    Emit a 'row-changed' signal when bookmark's properties emit a 'notify' event.
-    
-    @param bookmark: Bookmark that emitted 'notify' event.
+    Emit a 'row-changed' signal when bookmark's properties change.
+
+    @param bookmark: Bookmark whose properties changed.
     @type bookmark: L{BookmarkStore._Bookmark}
-    @param property: Property that changed, ignored because we emit dummy signals.
-    @type property: Property
     '''
     for row in self:
       if row[0] == bookmark:
@@ -185,7 +162,7 @@ class BookmarkStore(gtk.ListStore):
   def _getElements(self):
     '''
     Get a list of elements from XML doc. Filter out strings.
-    
+
     @return: list of elements.
     @rtype: list of Element
     '''
@@ -194,7 +171,7 @@ class BookmarkStore(gtk.ListStore):
   def _onRowChanged(self, model, tree_path, iter):
     '''
     Callback for row changes. Persist changes to disk.
-    
+
     @param model: Model that emitted signal
     @type model: L{BookmarkStore}
     @param path: Path of row that changed.
@@ -214,7 +191,7 @@ class BookmarkStore(gtk.ListStore):
   def _onRowDeleted(self, model, tree_path):
     '''
     Callback for row deletions. Persist changes to disk, and update UI.
-    
+
     @param model: Model that emitted signal
     @type model: L{BookmarkStore}
     @param path: Path of row that got deleted.
@@ -228,7 +205,7 @@ class BookmarkStore(gtk.ListStore):
   def _onRowInserted(self, model, path, iter):
     '''
     Callback for row insertions. Persist changes to disk.
-    
+
     @param model: Model that emitted signal
     @type model: L{BookmarkStore}
     @param path: Path of row that is inserted.
@@ -257,7 +234,7 @@ class BookmarkStore(gtk.ListStore):
   def _onBookmarkActivate(self, bookmark):
     '''
     Bookmark activation callback
-    
+
     @param bookmark: Bookmark that was activated.
     @type bookmark: L{BookmarkStore._Bookmark}
     '''
@@ -266,20 +243,20 @@ class BookmarkStore(gtk.ListStore):
   def jumpTo(self, bookmark):
     '''
     Go to bookmarks.
-    
-    @param bookmark: Bookmark to go to. 
+
+    @param bookmark: Bookmark to go to.
     @type bookmark: L{BookmarkStore._Bookmark}
     '''
-    if '' == bookmark.path: 
+    if '' == bookmark.path:
       path = ()
-    else: 
+    else:
       path = list(map(int, bookmark.path.split(',')))
     self.node.updateToPath(bookmark.app, path)
 
   def bookmarkCurrent(self):
     '''
     Bookmark the currently selected application-wide node.
-    
+
     @return: Tree iter of new bookmark.
     @rtype: gtk.TreeIter
     '''
@@ -301,16 +278,16 @@ class BookmarkStore(gtk.ListStore):
   def _nameIsTaken(self, name):
     '''
     Check if label text is already in use.
-    
+
     @param name: Name to check.
     @type name: string
-    
+
     @return: True is name is taken.
     @rtype: boolean
     '''
     for row in self:
       bookmark = row[0]
-      if bookmark.get_property('label') == name:
+      if bookmark.title == name:
         return True
     return False
 
@@ -321,16 +298,17 @@ class BookmarkStore(gtk.ListStore):
     def __init__(self, bookmarks_store):
       '''
       Initialize dialog.
-      
+
       @param bookmarks_store: Bookmarks manager.
       @type bookmarks_store: L{BookmarkStore}
       '''
       gtk.Dialog.__init__(self, name=_('Edit Bookmarks…'))
-      self.add_buttons(gtk.STOCK_CLOSE, gtk.ResponseType.CLOSE)
+      close_button = self.add_button(_("_Close"), gtk.ResponseType.CLOSE)
+      close_button.set_image(gtk.Image.new_from_icon_name("window-close", gtk.IconSize.BUTTON))
       self.set_default_size(480, 240)
       self.connect('response', self._onResponse)
       vbox = self.get_children()[0]
-      hbox = gtk.HBox()
+      hbox = gtk.Box()
       hbox.set_spacing(3)
       tv = self._createTreeView(bookmarks_store)
       sw = gtk.ScrolledWindow()
@@ -338,21 +316,21 @@ class BookmarkStore(gtk.ListStore):
       sw.set_shadow_type(gtk.ShadowType.IN)
       sw.add(tv)
       hbox.pack_start(sw, True, True, 0)
-      button_vbox = gtk.VBox()
+      button_vbox = gtk.Box(orientation=gtk.Orientation.VERTICAL)
       hbox.pack_start(button_vbox, False, False, 0)
-      add_button = gtk.Button.new_from_stock('gtk-add')
-      add_button.set_use_stock(True)
+      add_button = gtk.Button.new_with_mnemonic(_('_Add'))
+      add_button.set_image(gtk.Image.new_from_icon_name("list-add", gtk.IconSize.BUTTON))
       add_button.connect('clicked', self._onAddClicked, tv)
-      remove_button = gtk.Button.new_from_stock('gtk-remove')
-      remove_button.set_use_stock(True)
+      remove_button = gtk.Button.new_with_mnemonic(_('_Remove'))
+      remove_button.set_image(gtk.Image.new_from_icon_name("list-remove", gtk.IconSize.BUTTON))
       remove_button.connect('clicked', self._onRemoveClicked, tv)
-      jump_button = gtk.Button.new_from_stock('gtk-jump-to')
-      jump_button.set_use_stock(True)
+      jump_button = gtk.Button.new_with_mnemonic(_('_Jump to'))
+      jump_button.set_image(gtk.Image.new_from_icon_name("go-jump", gtk.IconSize.BUTTON))
       jump_button.connect('clicked', self._onJumpToClicked, tv)
       button_vbox.pack_start(add_button, False, False, 0)
       button_vbox.pack_start(remove_button, False, False, 0)
       button_vbox.pack_start(jump_button, False, False, 0)
-      vbox.add(hbox)
+      vbox.pack_start(hbox, True, True, 0)
       hbox.set_border_width(3)
       self.set_transient_for(bookmarks_store.parent_window)
       self.show_all()
@@ -360,7 +338,7 @@ class BookmarkStore(gtk.ListStore):
     def _onAddClicked(self, button, tv):
       '''
       Callback for add button. Add a bookmark.
-      
+
       @param button: Add button
       @type button: gtk.Button
       @param tv: Treeview of dialog.
@@ -375,7 +353,7 @@ class BookmarkStore(gtk.ListStore):
     def _onRemoveClicked(self, button, tv):
       '''
       Callback for remove button. Remove a bookmark.
-      
+
       @param button: Remove button
       @type button: gtk.Button
       @param tv: Treeview of dialog.
@@ -388,12 +366,12 @@ class BookmarkStore(gtk.ListStore):
         bookmark = model[iter][0]
         model.removeBookmark(bookmark)
         selection.select_path(0)
-        
+
 
     def _onJumpToClicked(self, button, tv):
       '''
       Callback for "jump to" button. Go to bookmark.
-      
+
       @param button: "jump to" button
       @type button: gtk.Button
       @param tv: Treeview of dialog.
@@ -407,7 +385,7 @@ class BookmarkStore(gtk.ListStore):
     def _onResponse(self, dialog, response):
       '''
       Callback for dialog response.
-      
+
       @param dialog: Dialog.
       @type dialog: gtk.Dialog
       @param response: response ID.
@@ -418,10 +396,10 @@ class BookmarkStore(gtk.ListStore):
     def _createTreeView(self, model):
       '''
       Create dialog's tree view.
-      
+
       @param model: Data model for view.
       @type model: L{BookmarkStore}
-      
+
       @return: The new tree view.
       @rtype: gtk.TreeView
       '''
@@ -456,8 +434,8 @@ class BookmarkStore(gtk.ListStore):
 
     def _onCellEdited(self, cellrenderer, path, new_text, model, col_id):
       '''
-      Callback for cell editing. Blocks unallowed input. 
-      
+      Callback for cell editing. Blocks unallowed input.
+
       @param cellrenderer: Cellrenderer that is being edited.
       @type cellrenderer: gtk.CellRendererText
       @param path: Path of tree node.
@@ -482,7 +460,7 @@ class BookmarkStore(gtk.ListStore):
     def _cellDataFunc(self, column, cell, model, iter, col_id):
       '''
       Cell renderer display function.
-      
+
       @param column: Tree view column
       @type column: gtk.TreeViewColumn
       @param cell: Cell renderer.
@@ -495,7 +473,7 @@ class BookmarkStore(gtk.ListStore):
       @type col_id: integer
       '''
       bookmark = model[iter][0]
-      cell.set_property('text', 
+      cell.set_property('text',
                         getattr(bookmark, _BM_ATTRIBS[col_id], ''))
 
   class _NewBookmarkDialog(gtk.Dialog):
@@ -512,31 +490,33 @@ class BookmarkStore(gtk.ListStore):
     def __init__(self, bookmark, parent_window):
       '''
       Initialize the dialog.
-      
+
       @param bookmark: New bookmark to edit.
       @type bookmark: L{BookmarkStore._Bookmark}
       '''
       gtk.Dialog.__init__(self, _('Add Bookmark…'))
-      self.add_button(gtk.STOCK_CANCEL, gtk.ResponseType.CANCEL)
-      ok_button = self.add_button(gtk.STOCK_ADD, gtk.ResponseType.OK)
+      cancel_button = self.add_button("_Cancel", gtk.ResponseType.CANCEL)
+      cancel_button.set_image(gtk.Image.new_from_icon_name("gtk-cancel", gtk.IconSize.BUTTON))
+      ok_button = self.add_button("_Add", gtk.ResponseType.OK)
+      ok_button.set_image(gtk.Image.new_from_icon_name("list-add", gtk.IconSize.BUTTON))
       ok_button.set_sensitive(False)
       self.set_default_response(gtk.ResponseType.OK)
-      table = gtk.Table.new(3, 2, False)
-      table.set_row_spacings(3)
-      table.set_col_spacings(3)
+      grid = gtk.Grid()
+      grid.set_row_spacing(3)
+      grid.set_column_spacing(3)
       vbox = self.get_children()[0]
-      vbox.add(table)
+      vbox.add(grid)
       self._title_entry = gtk.Entry()
       self._title_entry.connect('changed', self._onChanged, ok_button)
       self._app_entry = gtk.Entry()
       self._path_entry = gtk.Entry()
-      for i, label_entry_pair in enumerate([(_('Title:'), 
+      for i, label_entry_pair in enumerate([(_('Title:'),
                                              bookmark.title,
                                             self._title_entry),
-                                           (_('Application:'), 
+                                           (_('Application:'),
                                              bookmark.app,
                                             self._app_entry),
-                                           (_('Path:'), 
+                                           (_('Path:'),
                                              bookmark.path,
                                             self._path_entry)]):
         label, value, entry = label_entry_pair
@@ -548,15 +528,15 @@ class BookmarkStore(gtk.ListStore):
         entry_acc = entry.get_accessible()
         label_acc.add_relationship(atk.RelationType.LABEL_FOR, entry_acc)
         entry_acc.add_relationship(atk.RelationType.LABELLED_BY, label_acc)
-        table.attach(gtk.Label.new(label), 0, 1, i, i+1, gtk.AttachOptions.FILL, 0)
-        table.attach(entry, 1, 2, i, i+1)
+        grid.attach(gtk.Label.new(label), 0, i, 1, 1)
+        grid.attach(entry, 1, i, 1, 1)
       self.set_transient_for(parent_window)
       self.show_all()
 
     def _onChanged(self, entry, button):
       '''
       Title entry changed callback. If title entry is empty, disable "add" button.
-      
+
       @param entry: Entry widget that changed.
       @type entry: gtk.Entry
       @param button: Add button.
@@ -569,7 +549,7 @@ class BookmarkStore(gtk.ListStore):
     def _onEnter(self, entry, button):
       '''
       Finish dialog when enter is pressed.
-      
+
       @param entry: Entry widget that changed.
       @type entry: gtk.Entry
       @param button: Add button.
@@ -580,7 +560,7 @@ class BookmarkStore(gtk.ListStore):
     def getFields(self):
       '''
       Return value of all fields.
-      
+
       @return: title, app name, and path.
       @rtype: tuple
       '''
@@ -589,57 +569,110 @@ class BookmarkStore(gtk.ListStore):
           self._app_entry.get_text(), \
           self._path_entry.get_text()
 
-  class _Bookmark(gtk.Action):
+  class _Bookmark:
     '''
     Bookmark object.
 
+    @ivar bookmark_store: The bookmark store managing this bookmark.
+    @type bookmark_store: BookmarkStore
     @ivar title: Bookmark title (and label).
     @type title: string
     @ivar app: Application name.
     @type app: string
     @ivar path: Accessible path.
     @type path: string
-    @ivar merge_id: Merge id of UIManager.
-    @type merge_id: integer
+    @ivar bookmark_name: Name of the bookmark.
+    @type bookmark_name: string
+    @ivar accerciser_app: The accerciser application.
+    @type accerciser_app: gtk.Application
+    @ivar menu_item: The menu item for this bookmark.
+    @type menu_item: gio.MenuItem
     '''
-    def __init__(self, name, title, app, path, merge_id):
+    def __init__(self, bookmark_store, title, app, path, accerciser_app):
       '''
       Initialize bookmark.
-      
-      @param name: Action name
-      @type name: string
+
+      @param bookmark_store: The bookmark store managing this bookmark.
+      @type bookmark_store: BookmarkStore
       @param title: Bookmark title (and label).
       @type title: string
       @param app: Application name.
       @type app: string
       @param path: Accessible path.
       @type path: string
-      @param merge_id: Merge id of UIManager.
-      @type merge_id: integer
+      @param accerciser_app: The accerciser application.
+      @type accerciser_app: gtk.Application
       '''
-      gtk.Action.__init__(self, name, title, None, None)
+      self.accerciser_app = accerciser_app
+      # find an unused bookmark/action ID/name
+      id = 0
+      self.bookmark_name = f'bookmark{id}'
+      while self.accerciser_app.lookup_action(self.bookmark_name):
+        id = id +1
+        self.bookmark_name = f'bookmark{id}'
+
+      action_name = 'app.' + self.bookmark_name
+      self.menu_item = gio.MenuItem.new(title, action_name)
+      menus.bookmarks_menu_individual_section.append_item(self.menu_item)
+      action = gio.SimpleAction.new(self.bookmark_name, None)
+      action.connect('activate', self._onActivate)
+      self.accerciser_app.add_action(action)
+      self.bookmark_store = bookmark_store
       self._title = title
       self._app = app
       self._path = path
-      self.merge_id = merge_id
 
     def _getTitle(self):
       return self._title
     def _setTitle(self, title):
       self._title = title
-      self.set_property('label', title)
+      self.menu_item.set_attribute_value('label', GLib.Variant.new_string(title))
+      # When inserted into the menu, the menu item's data were just copied, so updating
+      # the menu item by itself wouldn't have any effect in the actual menu.
+      # Therefore, remove the current entry and insert anew.
+      index = self.getMenuItemIndex()
+      menus.bookmarks_menu_individual_section.remove(index)
+      menus.bookmarks_menu_individual_section.insert_item(index, self.menu_item)
+      self.bookmark_store._onBookmarkChanged(self)
     title = property(_getTitle, _setTitle)
 
     def _getApp(self):
       return self._app
     def _setApp(self, app):
       self._app = app
-      self.notify('name')
+      self.bookmark_store._onBookmarkChanged(self)
     app = property(_getApp, _setApp)
 
     def _getPath(self):
       return self._path
     def _setPath(self, path):
       self._path = path
-      self.notify('name')
+      self.bookmark_store._onBookmarkChanged(self)
     path = property(_getPath, _setPath)
+
+    def _onActivate(self, obj, data=None):
+      self.bookmark_store._onBookmarkActivate(self)
+
+    def getMenuItemIndex(self):
+      '''
+      Return the index that the bookmark's menu entry has in the menu.
+      '''
+      # iterate over the menu items to find the right one (match by action name)
+      index = -1
+      for i in range(0, menus.bookmarks_menu_individual_section.get_n_items()):
+        action_name = menus.bookmarks_menu_individual_section.get_item_attribute_value(i, 'action', None).get_string()
+        if action_name == 'app.' + self.bookmark_name:
+          index = i
+          break
+
+      assert(index >= 0)
+      return index
+
+    def removeSelf(self):
+      '''
+      Remove the bookmark's menu entry and action.
+      '''
+      # remove menu item from menu and remove the action
+      index = self.getMenuItemIndex()
+      menus.bookmarks_menu_individual_section.remove(index)
+      self.accerciser_app.remove_action(self.bookmark_name)
