@@ -6,8 +6,8 @@ Defines the preferences dialog.
 @copyright: Copyright (c) 2006, 2007 Mozilla Foundation
 @license: BSD
 
-All rights reserved. This program and the accompanying materials are made 
-available under the terms of the BSD which accompanies this distribution, and 
+All rights reserved. This program and the accompanying materials are made
+available under the terms of the BSD which accompanies this distribution, and
 is available at U{http://www.opensource.org/licenses/bsd-license.php}
 '''
 
@@ -29,14 +29,15 @@ class AccerciserPreferencesDialog(gtk.Dialog):
   def __init__(self, plugins_view=None, hotkeys_view=None):
     '''
     Initialize a preferences dialog.
-    
+
     @param plugins_view: Treeview of plugins.
     @type plugins_view: L{PluginManager._View}
     @param hotkeys_view: Treeview of global hotkeys.
     @type hotkeys_view: L{HotkeyTreeView}
     '''
     gtk.Dialog.__init__(self, title=_('accerciser Preferences'))
-    self.add_buttons(gtk.STOCK_CLOSE, gtk.ResponseType.CLOSE)
+    close_button = self.add_button(_('_Close'), gtk.ResponseType.CLOSE)
+    close_button.set_image(gtk.Image.new_from_icon_name('window-close', gtk.IconSize.BUTTON))
     self.connect('response', self._onResponse)
     self.set_default_size(500, 250)
     notebook = gtk.Notebook()
@@ -51,13 +52,13 @@ class AccerciserPreferencesDialog(gtk.Dialog):
         sw.set_size_request(500, 150)
         sw.add(view)
         notebook.append_page(sw, gtk.Label.new(section))
-    
+
     notebook.append_page(_HighlighterView(), gtk.Label.new(_('Highlighting')))
-        
+
   def _onResponse(self, dialog, response_id):
     '''
     Callback for dialog responses, always destroy it.
-    
+
     @param dialog: This dialog.
     @type dialog: L{AccerciserPreferencesDialog}
     @param response_id: Response ID recieved.
@@ -65,13 +66,17 @@ class AccerciserPreferencesDialog(gtk.Dialog):
     '''
     dialog.destroy()
 
-class _HighlighterView(gtk.Alignment):
+class _HighlighterView(gtk.Grid):
   '''
   A container widget with the settings for the highlighter.
   '''
   def __init__(self):
-    gtk.Alignment.__init__(self)
-    self.set_padding(12, 12, 18, 12)
+    gtk.Grid.__init__(self)
+    self.set_margin_top(12)
+    self.set_margin_bottom(12)
+    self.set_margin_start(18)
+    self.set_margin_end(12)
+    self.set_column_spacing(6)
     self.gsettings = GSettings.new('org.a11y.Accerciser')
     self._buildUI()
 
@@ -79,9 +84,6 @@ class _HighlighterView(gtk.Alignment):
     '''
     Programatically build the UI.
     '''
-    table = gtk.Table.new(3, 2, True)
-    table.set_col_spacings(6)
-    self.add(table)
     labels = [None, None, None]
     controls = [None, None, None]
     labels[0] = gtk.Label.new(_('Highlight duration:'))
@@ -102,8 +104,8 @@ class _HighlighterView(gtk.Alignment):
 
     for label, control, row in zip(labels, controls, range(3)):
       label.set_alignment(0, 0.5)
-      table.attach(label, 0, 1, row, row + 1, gtk.AttachOptions.FILL)
-      table.attach(control, 1, 2, row, row + 1, gtk.AttachOptions.FILL)
+      self.attach(label, 0, row, 1, 1)
+      self.attach(control, 1, row, 1, 1)
 
     for label, control in zip([x.get_accessible() for x in labels],
                               [x.get_accessible() for x in controls]):
@@ -121,7 +123,7 @@ class _HighlighterView(gtk.Alignment):
     node.HL_DURATION = int(spin_button.get_value()*1000)
     self.gsettings.set_double('highlight-duration',
                             spin_button.get_value())
-                            
+
 
   def _onColorSet(self, color_button, key):
     '''
@@ -139,7 +141,7 @@ class _HighlighterView(gtk.Alignment):
     else:
       node.BORDER_COLOR = color_button.get_rgb_string()
       node.BORDER_ALPHA = color_button.get_alpha_float()
-      
+
     self.gsettings.set_string(key, color_button.get_rgba_string())
 
   class _ColorButton(gtk.ColorButton):
@@ -152,7 +154,7 @@ class _HighlighterView(gtk.Alignment):
       self.set_use_alpha(True)
       self.set_alpha(int(alpha*0xffff))
       self.set_color(color)
-                               
+
     def get_rgba_string(self):
       '''
       Get the current color and alpha in string format.
@@ -183,7 +185,7 @@ class _HighlighterView(gtk.Alignment):
       color_val |= color.blue >> 8
       return \
           '#' + hex(color_val).replace('0x', '').replace('L', '').rjust(6, '0')
-      
+
     def get_alpha_float(self):
       '''
       Get the current alpha as a value from 0.0 to 1.0.
